@@ -8,23 +8,7 @@
  ******************************************************************************
  * Copyright (c) 2009, Phil Vachon, <philippe at cowpig.ca>
  *
- * Permission is hereby granted, free of charge, to any person obtaining a
- * copy of this software and associated documentation files (the "Software"),
- * to deal in the Software without restriction, including without limitation
- * the rights to use, copy, modify, merge, publish, distribute, sublicense,
- * and/or sell copies of the Software, and to permit persons to whom the
- * Software is furnished to do so, subject to the following conditions:
- *
- * The above copyright notice and this permission notice shall be included
- * in all copies or substantial portions of the Software.
- *
- * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
- * OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
- * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL
- * THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
- * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
- * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
- * DEALINGS IN THE SOFTWARE.
+ * SPDX-License-Identifier: MIT
  ****************************************************************************/
 
 #ifndef GDAL_PRIV_TEMPLATES_HPP_INCLUDED
@@ -32,6 +16,7 @@
 
 #include "cpl_port.h"
 
+#include <cmath>
 #include <cstdint>
 #include <limits>
 
@@ -54,13 +39,13 @@ inline void GDALGetDataLimits(Tin &tMaxValue, Tin &tMinValue)
     tMinValue = std::numeric_limits<Tin>::min();
 
     // Compute the actual minimum value of Tout in terms of Tin.
-    if (std::numeric_limits<Tout>::is_signed &&
-        std::numeric_limits<Tout>::is_integer)
+    if constexpr (std::numeric_limits<Tout>::is_signed &&
+                  std::numeric_limits<Tout>::is_integer)
     {
         // the minimum value is less than zero
-        if (std::numeric_limits<Tout>::digits <
-                std::numeric_limits<Tin>::digits ||
-            !std::numeric_limits<Tin>::is_integer)
+        if constexpr (std::numeric_limits<Tout>::digits <
+                          std::numeric_limits<Tin>::digits ||
+                      !std::numeric_limits<Tin>::is_integer)
         {
             // Tout is smaller than Tin, so we need to clamp values in input
             // to the range of Tout's min/max values
@@ -71,12 +56,12 @@ inline void GDALGetDataLimits(Tin &tMaxValue, Tin &tMinValue)
             tMaxValue = static_cast<Tin>(std::numeric_limits<Tout>::max());
         }
     }
-    else if (std::numeric_limits<Tout>::is_integer)
+    else if constexpr (std::numeric_limits<Tout>::is_integer)
     {
         // the output is unsigned, so we just need to determine the max
         /* coverity[same_on_both_sides] */
-        if (std::numeric_limits<Tout>::digits <=
-            std::numeric_limits<Tin>::digits)
+        if constexpr (std::numeric_limits<Tout>::digits <=
+                      std::numeric_limits<Tin>::digits)
         {
             // Tout is smaller than Tin, so we need to clamp the input values
             // to the range of Tout's max
@@ -148,16 +133,67 @@ template <class T> inline bool GDALIsValueInRange(double dfValue)
 
 template <> inline bool GDALIsValueInRange<double>(double dfValue)
 {
-    return !CPLIsNan(dfValue);
+    return !std::isnan(dfValue);
 }
 
 template <> inline bool GDALIsValueInRange<float>(double dfValue)
 {
-    return CPLIsInf(dfValue) ||
+    return std::isinf(dfValue) ||
            (dfValue >= -std::numeric_limits<float>::max() &&
             dfValue <= std::numeric_limits<float>::max());
 }
 
+template <> inline bool GDALIsValueInRange<int64_t>(double dfValue)
+{
+    // Values in the range [INT64_MAX - 1023, INT64_MAX - 1]
+    // get converted to a double that once cast to int64_t is
+    // INT64_MAX + 1, hence the < strict comparison.
+    return dfValue >=
+               static_cast<double>(std::numeric_limits<int64_t>::min()) &&
+           dfValue < static_cast<double>(std::numeric_limits<int64_t>::max());
+}
+
+template <> inline bool GDALIsValueInRange<uint64_t>(double dfValue)
+{
+    // Values in the range [UINT64_MAX - 2047, UINT64_MAX - 1]
+    // get converted to a double that once cast to uint64_t is
+    // UINT64_MAX + 1, hence the < strict comparison.
+    return dfValue >= 0 &&
+           dfValue < static_cast<double>(std::numeric_limits<uint64_t>::max());
+}
+
+/************************************************************************/
+/*                         GDALIsValueExactAs()                         */
+/************************************************************************/
+/**
+ * Returns whether a value can be exactly represented on type T.
+ *
+ * That is static_cast\<double\>(static_cast\<T\>(dfValue)) is legal and is
+ * equal to dfValue.
+ *
+ * Note: for T=float or double, a NaN input leads to true
+ *
+ * @param dfValue the value
+ * @return whether the value can be exactly represented on type T.
+ */
+template <class T> inline bool GDALIsValueExactAs(double dfValue)
+{
+    return GDALIsValueInRange<T>(dfValue) &&
+           static_cast<double>(static_cast<T>(dfValue)) == dfValue;
+}
+
+template <> inline bool GDALIsValueExactAs<float>(double dfValue)
+{
+    return std::isnan(dfValue) ||
+           (GDALIsValueInRange<float>(dfValue) &&
+            static_cast<double>(static_cast<float>(dfValue)) == dfValue);
+}
+
+template <> inline bool GDALIsValueExactAs<double>(double)
+{
+    return true;
+}
+
 /************************************************************************/
 /*                          GDALCopyWord()                              */
 /************************************************************************/
@@ -236,7 +272,7 @@ template <class Tout> struct sGDALCopyWord<float, Tout>
 {
     static inline void f(const float fValueIn, Tout &tValueOut)
     {
-        if (CPLIsNan(fValueIn))
+        if (std::isnan(fValueIn))
         {
             tValueOut = 0;
             return;
@@ -252,7 +288,7 @@ template <> struct sGDALCopyWord<float, short>
 {
     static inline void f(const float fValueIn, short &nValueOut)
     {
-        if (CPLIsNan(fValueIn))
+        if (std::isnan(fValueIn))
         {
             nValueOut = 0;
             return;
@@ -269,7 +305,7 @@ template <> struct sGDALCopyWord<float, signed char>
 {
     static inline void f(const float fValueIn, signed char &nValueOut)
     {
-        if (CPLIsNan(fValueIn))
+        if (std::isnan(fValueIn))
         {
             nValueOut = 0;
             return;
@@ -286,7 +322,7 @@ template <class Tout> struct sGDALCopyWord<double, Tout>
 {
     static inline void f(const double dfValueIn, Tout &tValueOut)
     {
-        if (CPLIsNan(dfValueIn))
+        if (std::isnan(dfValueIn))
         {
             tValueOut = 0;
             return;
@@ -302,7 +338,7 @@ template <> struct sGDALCopyWord<double, int>
 {
     static inline void f(const double dfValueIn, int &nValueOut)
     {
-        if (CPLIsNan(dfValueIn))
+        if (std::isnan(dfValueIn))
         {
             nValueOut = 0;
             return;
@@ -319,16 +355,45 @@ template <> struct sGDALCopyWord<double, std::int64_t>
 {
     static inline void f(const double dfValueIn, std::int64_t &nValueOut)
     {
-        if (CPLIsNan(dfValueIn))
+        if (std::isnan(dfValueIn))
         {
             nValueOut = 0;
-            return;
         }
-        double dfMaxVal, dfMinVal;
-        GDALGetDataLimits<double, std::int64_t>(dfMaxVal, dfMinVal);
-        double dfValue = dfValueIn >= 0.0 ? dfValueIn + 0.5 : dfValueIn - 0.5;
-        nValueOut = static_cast<std::int64_t>(
-            GDALClampValue(dfValue, dfMaxVal, dfMinVal));
+        else if (dfValueIn >=
+                 static_cast<double>(std::numeric_limits<std::int64_t>::max()))
+        {
+            nValueOut = std::numeric_limits<std::int64_t>::max();
+        }
+        else if (dfValueIn <=
+                 static_cast<double>(std::numeric_limits<std::int64_t>::min()))
+        {
+            nValueOut = std::numeric_limits<std::int64_t>::min();
+        }
+        else
+        {
+            nValueOut = static_cast<std::int64_t>(
+                dfValueIn > 0.0f ? dfValueIn + 0.5f : dfValueIn - 0.5f);
+        }
+    }
+};
+
+template <> struct sGDALCopyWord<double, std::uint64_t>
+{
+    static inline void f(const double dfValueIn, std::uint64_t &nValueOut)
+    {
+        if (!(dfValueIn > 0))
+        {
+            nValueOut = 0;
+        }
+        else if (dfValueIn >
+                 static_cast<double>(std::numeric_limits<uint64_t>::max()))
+        {
+            nValueOut = std::numeric_limits<uint64_t>::max();
+        }
+        else
+        {
+            nValueOut = static_cast<std::uint64_t>(dfValueIn + 0.5);
+        }
     }
 };
 
@@ -336,7 +401,7 @@ template <> struct sGDALCopyWord<double, short>
 {
     static inline void f(const double dfValueIn, short &nValueOut)
     {
-        if (CPLIsNan(dfValueIn))
+        if (std::isnan(dfValueIn))
         {
             nValueOut = 0;
             return;
@@ -353,7 +418,7 @@ template <> struct sGDALCopyWord<double, signed char>
 {
     static inline void f(const double dfValueIn, signed char &nValueOut)
     {
-        if (CPLIsNan(dfValueIn))
+        if (std::isnan(dfValueIn))
         {
             nValueOut = 0;
             return;
@@ -372,7 +437,12 @@ template <> struct sGDALCopyWord<float, int>
 {
     static inline void f(const float fValueIn, int &nValueOut)
     {
-        if (fValueIn >= static_cast<float>(std::numeric_limits<int>::max()))
+        if (std::isnan(fValueIn))
+        {
+            nValueOut = 0;
+        }
+        else if (fValueIn >=
+                 static_cast<float>(std::numeric_limits<int>::max()))
         {
             nValueOut = std::numeric_limits<int>::max();
         }
@@ -395,16 +465,15 @@ template <> struct sGDALCopyWord<float, unsigned int>
 {
     static inline void f(const float fValueIn, unsigned int &nValueOut)
     {
-        if (fValueIn >=
-            static_cast<float>(std::numeric_limits<unsigned int>::max()))
+        if (!(fValueIn > 0))
+        {
+            nValueOut = 0;
+        }
+        else if (fValueIn >=
+                 static_cast<float>(std::numeric_limits<unsigned int>::max()))
         {
             nValueOut = std::numeric_limits<unsigned int>::max();
         }
-        else if (fValueIn <=
-                 static_cast<float>(std::numeric_limits<unsigned int>::min()))
-        {
-            nValueOut = std::numeric_limits<unsigned int>::min();
-        }
         else
         {
             nValueOut = static_cast<unsigned int>(fValueIn + 0.5f);
@@ -418,8 +487,12 @@ template <> struct sGDALCopyWord<float, std::int64_t>
 {
     static inline void f(const float fValueIn, std::int64_t &nValueOut)
     {
-        if (fValueIn >=
-            static_cast<float>(std::numeric_limits<std::int64_t>::max()))
+        if (std::isnan(fValueIn))
+        {
+            nValueOut = 0;
+        }
+        else if (fValueIn >=
+                 static_cast<float>(std::numeric_limits<std::int64_t>::max()))
         {
             nValueOut = std::numeric_limits<std::int64_t>::max();
         }
@@ -442,16 +515,15 @@ template <> struct sGDALCopyWord<float, std::uint64_t>
 {
     static inline void f(const float fValueIn, std::uint64_t &nValueOut)
     {
-        if (fValueIn >=
-            static_cast<float>(std::numeric_limits<std::uint64_t>::max()))
+        if (!(fValueIn > 0))
+        {
+            nValueOut = 0;
+        }
+        else if (fValueIn >=
+                 static_cast<float>(std::numeric_limits<std::uint64_t>::max()))
         {
             nValueOut = std::numeric_limits<std::uint64_t>::max();
         }
-        else if (fValueIn <=
-                 static_cast<float>(std::numeric_limits<std::uint64_t>::min()))
-        {
-            nValueOut = std::numeric_limits<std::uint64_t>::min();
-        }
         else
         {
             nValueOut = static_cast<std::uint64_t>(fValueIn + 0.5f);
@@ -609,6 +681,7 @@ inline void GDALCopy4Words(const float *pValueIn, GUInt16 *const pValueOut)
 #ifdef __AVX2__
 
 #include <immintrin.h>
+
 template <>
 inline void GDALCopy8Words(const float *pValueIn, GByte *const pValueOut)
 {
