@@ -8,23 +8,7 @@
  * Copyright (c) 2002, Frank Warmerdam
  * Copyright (c) 2008-2014, Even Rouault <even dot rouault at spatialys.com>
  *
- * Permission is hereby granted, free of charge, to any person obtaining a
- * copy of this software and associated documentation files (the "Software"),
- * to deal in the Software without restriction, including without limitation
- * the rights to use, copy, modify, merge, publish, distribute, sublicense,
- * and/or sell copies of the Software, and to permit persons to whom the
- * Software is furnished to do so, subject to the following conditions:
- *
- * The above copyright notice and this permission notice shall be included
- * in all copies or substantial portions of the Software.
- *
- * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
- * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
- * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.  IN NO EVENT SHALL
- * THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
- * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
- * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
- * DEALINGS IN THE SOFTWARE.
+ * SPDX-License-Identifier: MIT
  ****************************************************************************/
 
 #include "cpl_port.h"
@@ -115,25 +99,7 @@ GMLReader::GMLReader(
     bool bUseExpatParserPreferably,
     bool bInvertAxisOrderIfLatLong, bool bConsiderEPSGAsURN,
     GMLSwapCoordinatesEnum eSwapCoordinates, bool bGetSecondaryGeometryOption)
-    : m_bClassListLocked(false), m_nClassCount(0), m_papoClass(nullptr),
-      m_bLookForClassAtAnyLevel(false), m_pszFilename(nullptr),
-#ifndef HAVE_XERCES
-      bUseExpatReader(true),
-#else
-      bUseExpatReader(false),
-#endif
-      m_poGMLHandler(nullptr),
-#ifdef HAVE_XERCES
-      m_poSAXReader(nullptr), m_poCompleteFeature(nullptr),
-      m_GMLInputSource(nullptr), m_bEOF(false), m_bXercesInitialized(false),
-#endif
-#ifdef HAVE_EXPAT
-      oParser(nullptr), ppoFeatureTab(nullptr), nFeatureTabLength(0),
-      nFeatureTabIndex(0), nFeatureTabAlloc(0), pabyBuf(nullptr),
-#endif
-      fpGML(nullptr), m_bReadStarted(false), m_poState(nullptr),
-      m_poRecycledState(nullptr), m_bStopParsing(false),
-      // Experimental. Not publicly advertized. See commented doc in
+    :  // Experimental. Not publicly advertized. See commented doc in
       // drv_gml.html
       m_bFetchAllGeometries(
           CPLTestBool(CPLGetConfigOption("GML_FETCH_ALL_GEOMETRIES", "NO"))),
@@ -141,15 +107,10 @@ GMLReader::GMLReader(
       m_bConsiderEPSGAsURN(bConsiderEPSGAsURN),
       m_eSwapCoordinates(eSwapCoordinates),
       m_bGetSecondaryGeometryOption(bGetSecondaryGeometryOption),
-      m_pszGlobalSRSName(nullptr), m_bCanUseGlobalSRSName(false),
-      m_pszFilteredClassName(nullptr), m_nFilteredClassIndex(-1),
-      m_nHasSequentialLayers(-1),
       // Must be in synced in OGR_G_CreateFromGML(), OGRGMLLayer::OGRGMLLayer(),
       // and GMLReader::GMLReader().
       m_bFaceHoleNegative(
-          CPLTestBool(CPLGetConfigOption("GML_FACE_HOLE_NEGATIVE", "NO"))),
-      m_bSetWidthFlag(true), m_bReportAllAttributes(false),
-      m_bIsWFSJointLayer(false), m_bEmptyAsNull(true)
+          CPLTestBool(CPLGetConfigOption("GML_FACE_HOLE_NEGATIVE", "NO")))
 {
 #ifndef HAVE_XERCES
 #else
@@ -510,7 +471,8 @@ GMLFeature *GMLReader::NextFeatureExpat()
         return nullptr;
     }
 
-    if (fpGML == nullptr || m_bStopParsing || VSIFEofL(fpGML))
+    if (fpGML == nullptr || m_bStopParsing || VSIFEofL(fpGML) ||
+        VSIFErrorL(fpGML))
         return nullptr;
 
     nFeatureTabLength = 0;
@@ -525,7 +487,7 @@ GMLFeature *GMLReader::NextFeatureExpat()
 
         unsigned int nLen = static_cast<unsigned int>(
             VSIFReadL(pabyBuf, 1, PARSER_BUF_SIZE, fpGML));
-        nDone = VSIFEofL(fpGML);
+        nDone = nLen < PARSER_BUF_SIZE;
 
         // Some files, such as APT_AIXM.xml from
         // https://nfdc.faa.gov/webContent/56DaySub/2015-03-05/aixm5.1.zip
@@ -538,11 +500,12 @@ GMLFeature *GMLReader::NextFeatureExpat()
         {
             // Defer emission of the error message until we have to return
             // nullptr
-            m_osErrorMessage.Printf("XML parsing of GML file failed : %s "
-                                    "at line %d, column %d",
-                                    XML_ErrorString(XML_GetErrorCode(oParser)),
-                                    (int)XML_GetCurrentLineNumber(oParser),
-                                    (int)XML_GetCurrentColumnNumber(oParser));
+            m_osErrorMessage.Printf(
+                "XML parsing of GML file failed : %s "
+                "at line %d, column %d",
+                XML_ErrorString(XML_GetErrorCode(oParser)),
+                static_cast<int>(XML_GetCurrentLineNumber(oParser)),
+                static_cast<int>(XML_GetCurrentColumnNumber(oParser)));
             m_bStopParsing = true;
         }
         if (!m_bStopParsing)
@@ -716,19 +679,19 @@ int GMLReader::GetFeatureElementIndex(const char *pszElement,
         }
 
         // Begin of CSW SearchResults.
-        else if (nElementLength == (int)strlen("BriefRecord") &&
+        else if (nElementLength == static_cast<int>(strlen("BriefRecord")) &&
                  nLenLast == strlen("SearchResults") &&
                  strcmp(pszElement, "BriefRecord") == 0 &&
                  strcmp(pszLast, "SearchResults") == 0)
         {
         }
-        else if (nElementLength == (int)strlen("SummaryRecord") &&
+        else if (nElementLength == static_cast<int>(strlen("SummaryRecord")) &&
                  nLenLast == strlen("SearchResults") &&
                  strcmp(pszElement, "SummaryRecord") == 0 &&
                  strcmp(pszLast, "SearchResults") == 0)
         {
         }
-        else if (nElementLength == (int)strlen("Record") &&
+        else if (nElementLength == static_cast<int>(strlen("Record")) &&
                  nLenLast == strlen("SearchResults") &&
                  strcmp(pszElement, "Record") == 0 &&
                  strcmp(pszLast, "SearchResults") == 0)
@@ -778,7 +741,8 @@ int GMLReader::GetFeatureElementIndex(const char *pszElement,
     // otherwise, find a class with the desired element name.
     for (int i = 0; i < m_nClassCount; i++)
     {
-        if (nElementLength == (int)m_papoClass[i]->GetElementNameLen() &&
+        if (nElementLength ==
+                static_cast<int>(m_papoClass[i]->GetElementNameLen()) &&
             memcmp(pszElement, m_papoClass[i]->GetElementName(),
                    nElementLength) == 0)
             return i;
@@ -907,8 +871,8 @@ void GMLReader::PopState()
             if (nFeatureTabLength >= nFeatureTabAlloc)
             {
                 nFeatureTabAlloc = nFeatureTabLength * 4 / 3 + 16;
-                ppoFeatureTab = (GMLFeature **)CPLRealloc(
-                    ppoFeatureTab, sizeof(GMLFeature *) * (nFeatureTabAlloc));
+                ppoFeatureTab = static_cast<GMLFeature **>(CPLRealloc(
+                    ppoFeatureTab, sizeof(GMLFeature *) * (nFeatureTabAlloc)));
             }
             ppoFeatureTab[nFeatureTabLength] = m_poState->m_poFeature;
             nFeatureTabLength++;
